# audio codec dialogs for the BiaMovE application

import Tkinter
import tkFileDialog
import tkSimpleDialog
import tkMessageBox

class FAACDialog (tkSimpleDialog.Dialog):
	""" FAAC audio codec options dialog"""
	
	def apply (self):
		self.result = "-faacopts "
		if self.bitrateorqualityflag.get() == "bitrateflag":
			self.result += "br=" + self.bitrate.get()
		else:
			self.result += "quality=" + self.quality.get()
		self.result += ":object=" + self.objecttype.get().split()[0] + ":mpeg=" + self.mpegversion.get()
		
	def body (self, master):
		Tkinter.Label (master, text="FAAC audio encoding options (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=2)
		
		self.bitrateorqualityflag = Tkinter.StringVar ()
		self.bitrateorqualityflag.set ("bitrateflag")
		
		Tkinter.Radiobutton (master, variable=self.bitrateorqualityflag, text="Use bitrate", value="bitrateflag").grid (column=0, row=1)
		Tkinter.Radiobutton (master, variable=self.bitrateorqualityflag, text="Use quality", value="qualityflag").grid (column=1, row=1)
		
		Tkinter.Label (master, text="Bitrate").grid (column=0, row=2)
		
		self.bitrate = Tkinter.StringVar ()
		self.bitrate.set (64)
		Tkinter.Spinbox (master, textvariable=self.bitrate, from_=0, to=1024).grid (column=1, row=2)
		
		Tkinter.Label (master, text="Quality (1 to 1000)").grid (column=0, row=3)
		
		self.quality = Tkinter.StringVar ()
		self.quality.set (500)
		Tkinter.Spinbox (master, textvariable=self.quality, from_=1, to=1000).grid (column=1, row=3)
		
		Tkinter.Label (master, text="Object type complexity").grid (column=0, row=4)
		
		self.objecttype = Tkinter.StringVar ()
		self.objecttype.set ("1   MAIN (default)")
		Tkinter.OptionMenu (master, self.objecttype, "1   MAIN (default)", "2   LOW", "3   SSR", "4   LTP (extremely slow)").grid (column=1, row=4)
		
		Tkinter.Label (master, text="MPEG version").grid (column=0, row=5)
		
		self.mpegversion = Tkinter.StringVar ()
		self.mpegversion.set ("4")
		Tkinter.OptionMenu (master, self.mpegversion, "2", "4").grid (column=1, row=5)

class TwolameDialog (tkSimpleDialog.Dialog):
	""" Twolame audio codec options dialog"""
	
	def apply (self):
		self.result = "-twolameopts "
		self.result += "br=" + self.bitrate.get()
		if self.variability.get() != "0":
			self.result += ":vbr=" + self.variability.get() + ":maxvbr=" + self.maxbitrate.get()
		self.result += ":mode=" + self.mode.get()
	
	def body(self, master):
		Tkinter.Label (master, text="Twolame audio encoding options (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=2)
		Tkinter.Label (master, text="Bitrate variability range (-50 to 50, 0 for CBR)").grid (column=0, row=1)
		
		self.variability = Tkinter.StringVar ()
		self.variability.set (0)
		Tkinter.Spinbox (master, textvariable = self.variability, from_=-50, to=50).grid (column=1, row=1)
		
		Tkinter.Label (master, text="Bitrate (min rate for VBR) (32 to 384)").grid (column=0, row=2)
		
		self.bitrate = Tkinter.StringVar ()
		self.bitrate.set (128)
		Tkinter.Spinbox (master, textvariable = self.bitrate, from_=32, to=384).grid (column=1, row=2)
		
		Tkinter.Label (master, text="Max bitrate (VBR only) (32 to 384)").grid (column=0, row=3)
		
		self.maxbitrate = Tkinter.StringVar ()
		self.maxbitrate.set (192)
		Tkinter.Spinbox (master, textvariable = self.maxbitrate, from_=32, to=384).grid (column=1, row=3)
		
		Tkinter.Label (master, text="Mode").grid (column=0, row=4)
		
		self.mode = Tkinter.StringVar ()
		self.mode.set ("stereo")
		Tkinter.OptionMenu (master, self.mode, "stereo", "jstereo", "mono", "dual").grid (column=1, row=4)

class LAMEDialog (tkSimpleDialog.Dialog):
	"""LAME Audio codec options dialog"""
	def apply (self):
		self.result = "-lameopts "
		if self.bitratemethod.get() == "vbr":
			self.result += "vbr=" + self.vbrmethod.get().split()[0]
			self.result += ":q=" + str (self.vbrquality.get())
		else:
			self.result += self.bitratemethod.get() + ":br=" + str(self.bitrate.get())

		self.result += ":vol=" + str(self.audiogain.get())
		
		if (self.audiomode.get() != "(auto)"):
			self.result += ":mode=" + self.audiomode.get().split()[0]
	
	def body (self, master):
		Tkinter.Label (master, text="LAME audio encoding options (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=3)
		
		self.bitratemethod = Tkinter.StringVar ()
		self.bitratemethod.set ("abr")
		
		bitratetypes = [ ("cbr", "Constant bitrate"), ("vbr", "Variable bitrate"), ("abr", "Average bitrate") ]
		i = 0
		for val, txt in bitratetypes:
			Tkinter.Radiobutton (master, variable=self.bitratemethod, text=txt, value=val).grid (column=i, row=1)
			i += 1
			
		Tkinter.Label (master, text="Variable bitrate options", fg="darkblue").grid (column=0, row=2, columnspan=3, pady=10)
		
		Tkinter.Label (master, text="VBR Method").grid (column=0, row=3)
				
		self.vbrmethod = Tkinter.StringVar ()
		self.vbrmethod.set ("2   rh")
		Tkinter.OptionMenu (master, self.vbrmethod, "0   cbr", "1   mt", "2   rh", "3   abr", "4   mtrh").grid (column=1, row=3)
		
		Tkinter.Label (master, text="VBR Quality (0-highest 9-lowest)").grid (column=0, row=4)
		
		self.vbrquality = Tkinter.StringVar ()
		self.vbrquality.set (0)
		Tkinter.Spinbox (master, textvariable=self.vbrquality, from_=0, to=9).grid (column=1, row=4)
		
		Tkinter.Label (master, text="Constant/average bitrate options", fg="darkblue").grid (column=0, row=5, columnspan=3, pady=10)
		
		Tkinter.Label (master, text="Bitrate (0 to 1024)").grid (column=0, row=6)
		
		self.bitrate = Tkinter.StringVar ()
		self.bitrate.set (128)
		Tkinter.Spinbox (master, textvariable=self.bitrate, from_=0, to=1024).grid (column=1, row=6)
		
		Tkinter.Label (master, text="General options", fg="darkblue").grid (column=0, row=7, columnspan=3, pady=10)
		
		Tkinter.Label (master, text="Audio input gain (0 to 10)").grid (column=0, row=8)
		
		self.audiogain = Tkinter.StringVar ()
		self.audiogain. set (0)
		Tkinter.Spinbox (master, textvariable = self.audiogain, from_=0, to=10).grid (column=1, row=8)
		
		Tkinter.Label (master, text="Mode").grid (column=0, row=9)
		
		self.audiomode = Tkinter.StringVar ()
		self.audiomode.set ("(auto)")
		Tkinter.OptionMenu (master, self.audiomode, "(auto)", "0   stereo", "1   joint-stereo", "2   dualchannel", "3   mono").grid (column=1, row=9)

class LAVCAudioDialog (tkSimpleDialog.Dialog):
	"""LAVC Video codec options dialog"""
	def apply (self):
		self.result = "-lavcopts acodec=" + self.audiocodecopt.get().split ()[0] + ":abitrate=" + str (self.audiobitrate.get())
	
	def body (self, master):
		Tkinter.Label (master, text="LAVC audio codec options (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=2)

		Tkinter.Label (master, text="Audio codec to use").grid (column=0, row=1)

		self.audiocodecopt = Tkinter.StringVar ()
		self.audiocodecopt.set ("mp2   MPEG-1 audio layer 2 (MP2)")
		Tkinter.OptionMenu (master, self.audiocodecopt, "ac3   Dolby Digital (AC-3)", "adpcm_ima_qt   Apple QuickTime", "adpcm_ima_wav   Microsoft/IBM WAVE", "adpcm_ima_dk3   Duck DK3", "adpcm_ima_dk4   Duck DK4", "adpcm_ima_ws   Westwood Studio", "adpcm_ima_smjpeg   SDL Motion JPEG", "adpcm_ms   Microsoft", "adpcm_4xm   4X Technologies", "adpcm_xa   Phillips Yellow Book CD-ROM eXtended Architecture", "adpcm_ea   Electronic Arts", "adpcm_ct   Creative 16->4-bit", "adpcm_swf   Adobe Shockwave Flash", "adpcm_yamaha   Yamaha", "adpcm_sbpro_4   Creative VOC SoundBlaster Pro 8->4-bit", "adpcm_sbpro_3   Creative VOC SoundBlaster Pro 8->2.6-bit", "adpcm_sbpro_2   Creative VOC SoundBlaster Pro 8->2-bit", "adpcm_thp   Nintendo GameCube FMV THP", "adpcm_adx   Sega/CRI ADX", "flac   Free Lossless Audio Codec (FLAC)", "g726   G.726 ADPCM", "libamr_nb   3GPP Adaptive Multi-Rate (AMR) narrow-band", "libamr_wb   3GPP Adaptive Multi-Rate (AMR) wide-band", "libfaac   Advanced Audio Coding (AAC) - using FAAC", "libgsm   ETSI GSM 06.10 full rate", "libgsm_ms   Microsoft GSM", "libmp3lame   MPEG-1 audio layer 3 (MP3) - using LAME", "mp2   MPEG-1 audio layer 2 (MP2)", "roq_dpcm   Id Software RoQ DPCM", "sonic   experimental FFmpeg lossy codec", "sonicls   experimental FFmpeg lossless codec", "vorbis   Vorbis", "wmav1   Windows Media Audio v1", "wmav2   Windows Media Audio v2", "pcm_s32le   signed 32-bit little-endian", "pcm_s32be   signed 32-bit big-endian", "pcm_u32le   unsigned 32-bit little-endian", "pcm_u32be   unsigned 32-bit big-endian", "pcm_s24le   signed 24-bit little-endian", "pcm_s24be   signed 24-bit big-endian", "pcm_u24le   unsigned 24-bit little-endian", "pcm_u24be   unsigned 24-bit big-endian", "pcm_s16le   signed 16-bit little-endian", "pcm_s16be   signed 16-bit big-endian", "pcm_u16le   unsigned 16-bit little-endian", "pcm_u16be   unsigned 16-bit big-endian", "pcm_s8   signed 8-bit", "pcm_u8   unsigned 8-bit", "pcm_alaw   G.711 A-LAW", "pcm_mulaw   G.711 u-LAW", "pcm_s24daud   signed 24-bit D-Cinema Audio format", "pcm_zork   Activision Zork Nemesis").grid (column=1, row=1)
		
		Tkinter.Label (master, text="Audio bitrate").grid (column=0, row=2)
		
		self.audiobitrate = Tkinter.StringVar ()
		self.audiobitrate.set (224)
		Tkinter.Spinbox (master, textvariable=self.audiobitrate, from_=0, to=1024).grid (column=1, row=2)
