# Main Window class for the BiaMovE application

import Tkinter
import tkFileDialog
import tkSimpleDialog
import tkMessageBox
import os
import os.path
import subprocess
import sys
import ConfigParser as CP

# import all the dialog classes
from audiocodecdialogs import *
from videofilterdialogs import *
from videocodecdialogs import *
from videomuxerdialogs import *
from miscdialogs import *

class MainWindow (Tkinter.Frame):
	"""The main window class of the BiaMovE application"""
	def on_misc_options (self):
		""" Called when the misc options button is clicked"""
		miscdlg = MiscOptionsDialog (self)
		if miscdlg.result != None:
			self.otherparameters.set (miscdlg.result)
		
	def on_videofilters (self):
		"""Called when the video filters button is clicked"""
		if self.outputvideo.get().split()[0] == "copy":
			tkMessageBox.showinfo ("Video filters", "You cannot apply filters when the video stream is directly copied. Change the video codec.")
		else:
			filtersdlg = VideoFiltersDialog (self)
			if filtersdlg.result != None:
				self.videofilters.set (filtersdlg.result)
	
	def build_mencoder_command (self):
		""" To build the mencoder command """
		
		# mencoder command line
		commandlist = ["mencoder",]
		
		# dvd options (if present)
		for item in self.dvdopts.get().split():
			commandlist.append (item)
			
		# output format
		commandlist.append ("-of")
		commandlist.append (self.outputcontainer.get().split()[0])
		
		for item in self.outputcontaineropts.get().split():
			commandlist.append (item)
		
		# video codec
		commandlist.append ("-ovc")
		commandlist.append (self.outputvideo.get().split()[0])
		
		for item in self.videoopts.get().split():
			commandlist.append (item)
		
		# audio codec
		commandlist.append ("-oac")
		commandlist.append (self.outputaudio.get().split()[0])
		
		for item in self.audioopts.get().split():
			commandlist.append (item)
			
		# video filters
		if self.videofilters.get().strip() != "":
			commandlist.append ("-vf")
			commandlist.append (self.videofilters.get().strip())
		
		# misc options
		for item in self.otherparameters.get().split():
			commandlist.append (item)
			
		# input file
		commandlist.append (self.inputfile.get())
		
		# output file
		commandlist.append ("-o")
		commandlist.append (self.outputfile.get())
			
		return commandlist
	
	def on_encode (self):
		"""Called when the encode button is clicked"""
		if self.inputfile.get().strip() == "" or self.outputfile.get().strip() == "":
			tkMessageBox.showerror ("Cannot encode", "No input and/or output file specified")
		else:
			commandlist = self.build_mencoder_command ()
			
			if tkMessageBox.askyesno ("Confirm", "Are you sure you wish to encode (in *NIX you must have TERM environment variable set)?"):
				if sys.platform.startswith ('win'):
					po, pi = os.popen2 ("cmd /K " + subprocess.list2cmdline (commandlist))
				else:
					if "TERM" in os.environ:
						if os.environ["TERM"] == "xterm":
							term_command = os.environ["TERM"] + " -hold -e " + subprocess.list2cmdline (commandlist)
						else:
							term_command = os.environ["TERM"] + " -e " + subprocess.list2cmdline(commandlist)
						po, pi = os.popen2 (term_command)
					else:
						tkMessageBox.showerror ("Cannot encode", "The TERM environment variable is not set. Cannot find a suitable terminal.")
				
	def on_save_profile (self):
		""" Called when the save profile button is clicked"""
		profile_name = tkSimpleDialog.askstring ("Save profile", "Give a name for this profile")
		if profile_name != None:
			config = CP.SafeConfigParser ()
			
			cfgfile = os.path.join (os.path.expanduser ("~"), ".biamoverc")
			
			config.read ( cfgfile )
			if (config.has_section (profile_name)):
				if not tkMessageBox.askyesno ("Section exists", "This section exists. Do you wish to overwrite the settings?"):
					return
			else:
				config.add_section (profile_name)
	
			config.set (profile_name, "ContainerFormat", self.outputcontainer.get())
			config.set (profile_name, "VideoCodec", self.outputvideo.get())
			config.set (profile_name, "AudioCodec", self.outputaudio.get())
			config.set (profile_name, "ContainerParameters", self.outputcontaineropts.get())
			config.set (profile_name, "VideoParameters", self.videoopts.get())
			config.set (profile_name, "AudioParameters", self.audioopts.get())
			config.set (profile_name, "VideoFilters", self.videofilters.get())
			config.set (profile_name, "OtherParameters", self.otherparameters.get())
						
			config.write (file (cfgfile, "w"))
	
	def on_load_profile (self):
		""" Called when the load profile button is clicked"""
		loaddlg = ProfileLoadDialog (self)
		sect = loaddlg.result
		if sect != None:
			config = CP.SafeConfigParser ()
			cfgfile = os.path.join (os.path.expanduser("~"), ".biamoverc")
			config.read (cfgfile)
			
			if (config.has_section (sect)):
				self.outputcontainer.set (config.get (sect, "ContainerFormat"))
				self.outputvideo.set (config.get (sect, "VideoCodec"))
				self.outputaudio.set (config.get (sect, "AudioCodec"))
				self.outputcontaineropts.set (config.get (sect, "ContainerParameters"))
				self.videoopts.set (config.get (sect, "VideoParameters"))
				self.audioopts.set (config.get (sect, "AudioParameters"))
				self.videofilters.set (config.get (sect, "VideoFilters"))
				self.otherparameters.set (config.get (sect, "OtherParameters"))

	def on_container_options (self):
		""" Called when the container options button is clicked"""
		containerfmt = self.outputcontainer.get().split()[0]
		if (containerfmt == "lavf"):
			lavfdlg = LAVFMuxerDialog (self)
			if lavfdlg.result != None:
				self.outputcontaineropts.set (lavfdlg.result)
		elif (containerfmt == "mpeg"):
			mpegdlg = MPEGMuxerDialog (self)
			if mpegdlg.result != None:
				self.outputcontaineropts.set (mpegdlg.result)
		else:
			tkMessageBox.showinfo ("Muxer options","No options for this muxer or options not implemented yet")
			self.outputcontaineropts.set ("")
	
	def on_codec_video (self):
		""" Called when the video codec options button is clicked"""
		videocodec = self.outputvideo.get().split()[0]
		if (videocodec == "nuv"):
			nuvdlg = NUVDialog (self)
			if nuvdlg.result != None:
				self.videoopts.set (nuvdlg.result)
		elif (videocodec == "lavc"):
			lavcdlg = LAVCVideoDialog (self)
			if lavcdlg.result != None:
				self.videoopts.set (lavcdlg.result)
		elif (videocodec == "xvid"):
			xviddlg = XvidEncDialog (self)
			if xviddlg.result != None:
				self.videoopts.set (xviddlg.result)
		elif (videocodec == "x264"):
			x264dlg = X264EncDialog (self)
			if x264dlg.result != None:
				self.videoopts.set (x264dlg.result)
		else:
			tkMessageBox.showinfo ("Video options", "No options for this codec or options not implemented yet")
			self.videoopts.set ("")
	
	def on_codec_audio (self):
		""" Called when the audio codec options button is clicked"""
		
		# if the output audio codec is lavc show the lavc codec dialog
		audiocodec = self.outputaudio.get().split()[0]
		if (audiocodec == "lavc"):
			lavcdlg = LAVCAudioDialog (self)
			if lavcdlg.result != None:
				self.audioopts.set (lavcdlg.result)
		# if the output audio codec is lame show the lame codec dialog
		elif (audiocodec == "mp3lame"):
			lamedlg = LAMEDialog (self)
			if lamedlg.result != None:
				self.audioopts.set (lamedlg.result)
		elif (audiocodec == "twolame"):
			twolamedlg = TwolameDialog (self)
			if twolamedlg.result != None:
				self.audioopts.set (twolamedlg.result)
		elif (audiocodec == "faac"):
			faacdlg = FAACDialog (self)
			if faacdlg.result != None:
				self.audioopts.set (faacdlg.result)
		else:
			tkMessageBox.showinfo ("Audio options", "No options for this codec or options not implemented yet")
			self.audioopts.set ("")
	
	def on_input_browse (self):
		"""Called when the input browse button is clicked"""
		infile = tkFileDialog.askopenfilename ()
		if infile != None:
			self.inputfile.set (infile)
	
	def on_output_browse (self):
		""" Called when the output browse button is clicked"""
		outfile = tkFileDialog.asksaveasfilename ()
		if outfile != None:
			self.outputfile.set (outfile)
			
	def on_dvd_device (self):
		""" called when the DVD options button is clicked"""
		dvddlg = DVDDialog (self)
		if dvddlg.result != None:
			self.inputfile.set (dvddlg.result)
			self.dvdopts.set (dvddlg.result_args)
	
	def create_widgets (self):
		""" Called during __init__ """
		Tkinter.Label (self, text="You need Mencoder installed (compiled with appropriate codecs) for this program to function properly. This program does NOT detect which codecs are available and which are not.", fg = "darkblue", wraplength=600).grid(column=0, row=0, columnspan=4)
		
		Tkinter.Label (self, text="Input file/source URL").grid (column=0, row=1)
		
		self.inputfile = Tkinter.StringVar ()
		Tkinter.Entry (self, textvariable=self.inputfile).grid (column=1, row=1)
		
		Tkinter.Button (self, text="browse...", command=self.on_input_browse).grid (column=2, row=1)
		
		dvdframe = Tkinter.Frame (self)
		
		self.dvdopts = Tkinter.StringVar ()
		
		Tkinter.Button (dvdframe, text="DVD...", command=self.on_dvd_device).pack (side=Tkinter.LEFT)
		Tkinter.Entry (dvdframe, textvariable=self.dvdopts).pack (side=Tkinter.RIGHT)
		
		dvdframe.grid (column=3, row=1, columnspan=2)
		
		Tkinter.Label (self, text="Output file").grid (column=0, row=2)
		
		self.outputfile = Tkinter.StringVar ()
		Tkinter.Entry (self, textvariable=self.outputfile).grid (column=1, row=2)
		
		Tkinter.Button (self, text="browse...", command=self.on_output_browse).grid (column=2, row=2)
		
		Tkinter.Label (self, text="mencoder options", fg="darkblue").grid (column=0, row=3, pady=5, columnspan=4)
		
		Tkinter.Label (self, text="Output container format").grid (column=0, row=4)
		
		self.outputcontainer = Tkinter.StringVar()
		self.outputcontainer.set ("avi   Microsoft AVI")
		Tkinter.OptionMenu (self, self.outputcontainer, "avi   Microsoft AVI", "mpeg   MPEG 1/2 stream", "lavf   FFMPEG libavformat muxers", "rawvideo   Raw video (only) stream", "rawaudio   Raw audio (only) stream").grid (column=1, row=4)
		
		Tkinter.Button (self, text="Container options...", command=self.on_container_options).grid (column=2, row=4)
		
		self.outputcontaineropts = Tkinter.StringVar ()
		Tkinter.Entry (self, textvariable=self.outputcontaineropts).grid (column=3, row=4)

		Tkinter.Label (self, text="Output video codec").grid (column=0, row=5)

		self.outputvideo = Tkinter.StringVar ()
		self.outputvideo.set ("copy   Frames copy")
		Tkinter.OptionMenu (self, self.outputvideo, "copy   Frames copy", "raw   Uncompressed video", "nuv   Nuppel video", "lavc   FFMPEG codecs ", "xvid   XviD encoding", "x264   H.264 encoding").grid (column=1, row=5)
		
		Tkinter.Button (self, text="codec options...", command=self.on_codec_video).grid (column=2, row=5)
		
		self.videoopts = Tkinter.StringVar ()
		Tkinter.Entry (self, textvariable=self.videoopts).grid (column=3, row=5)
		
		Tkinter.Label (self, text="Output audio codec").grid (column=0, row=6)
		
		self.outputaudio = Tkinter.StringVar ()
		self.outputaudio.set ("copy   Frames copy")
		Tkinter.OptionMenu (self, self.outputaudio, "copy   Frames copy", "pcm   Uncompressed PCM", "mp3lame   MP3 using LAME", "lavc   FFMPEG codecs", "twolame   TwoLAME MP2", "faac   FAAC AAC Audio").grid (column=1, row=6)
		
		Tkinter.Button (self, text="codec options...", command = self.on_codec_audio).grid (column=2, row=6)
		
		self.audioopts = Tkinter.StringVar ()
		Tkinter.Entry (self, textvariable = self.audioopts).grid (column=3, row=6)
		
		Tkinter.Label (self, text="Video filters").grid (column=0, row=7)
		Tkinter.Button (self, text="setup filters...", command=self.on_videofilters).grid (column=1, row=7)
		
		self.videofilters = Tkinter.StringVar ()
		Tkinter.Entry (self, textvariable=self.videofilters).grid (column=2, row=7)
		
		Tkinter.Label (self, text="Other Mencoder parameters").grid (column=0, row=8, pady=10)
		
		Tkinter.Button (self, text="Misc Options...", command=self.on_misc_options).grid (column=1, row=8)
		
		self.otherparameters = Tkinter.StringVar ()
		Tkinter.Entry (self, textvariable=self.otherparameters).grid (column=2, row=8)
		
		Tkinter.Label (self, text="Save/Load Profile", fg="darkblue").grid (column=0, row=9, pady=5, columnspan=2)
		
		leftframe = Tkinter.Frame (self)
		Tkinter.Button (leftframe, text="Save...", command=self.on_save_profile).pack (side=Tkinter.LEFT)
		Tkinter.Button (leftframe, text="Load...", command=self.on_load_profile).pack (side=Tkinter.RIGHT)
		leftframe.grid (column=0, row=10, columnspan=2)
		
		Tkinter.Label (self, text="Actions", fg="darkblue").grid (column=2, row=9, pady=5, columnspan=2)
		
		rightframe = Tkinter.Frame (self)
		Tkinter.Button (rightframe, text="Encode", command=self.on_encode).pack (side=Tkinter.LEFT)
		Tkinter.Button (rightframe, text="Quit", command=self.quit).pack (side=Tkinter.RIGHT)
		rightframe.grid (column=2, row=10, columnspan=2)
		
	def __init__ (self, master=None):
		Tkinter.Frame.__init__ (self, master)
		
		self.grid (padx=10,pady=10)
		self.create_widgets ()
