# Miscellaneous dialogs for the BiaMovE application

import Tkinter
import tkFileDialog
import tkSimpleDialog
import tkMessageBox
import os.path
import sys
import ConfigParser as CP

class DVDInfoDialog (tkSimpleDialog.Dialog):
	""" Class to display the DVD information (works only on *NIX and only if the biadvdlister
	executable program is installed """
	def __init__ (self, master, dvddevice):
		self.dvddevice = dvddevice
		tkSimpleDialog.Dialog.__init__ (self, master, "DVD Information")
	
	def fill_info (self, master):
		if self.dvddevice.strip() == "":
			self.dvddevice = "/dev/dvd"
		
		fi, fo, fe = os.popen3 ("biadvdlister --dvd-device " + self.dvddevice)
		infos = fo.readlines ()
		if infos == []:
			errors = fe.readlines ()
			i = 1
			for error in errors:
				Tkinter.Label (master, text=error.strip()).grid (column=0, row=i, columnspan=2)
				i += 1
		else:
			r = 1
			c = 0
			for info in infos:
				rec = info.split ()
				for field in rec:
					Tkinter.Label (master, text=field).grid (column=c, row=r)
					c += 1
				r += 1
				c = 0
	
	def body (self, master):
		Tkinter.Label (master, text="DVD information (you need the biadvdlister program installed for this feature)", fg="darkblue", wraplength=280).grid (column=0, row=0, columnspan=2)
		
		self.fill_info (master)

class DVDDialog (tkSimpleDialog.Dialog):
	""" Class to choose DVD device and set DVD options """
	def apply (self):
		self.result = "dvd://" + self.dvdtrack.get ()
		
		self.result_args = ""
		if self.allchapters.get() == 0:
			self.result_args += " -chapter " + self.startchapter.get()
			if self.endchapter.get().strip () != "":
				self.result_args += "-" + self.endchapter.get()
		if self.dvddevice.get().strip() != "":
			self.result_args += " -dvd-device " + self.dvddevice.get()
		
		self.result = self.result.strip ()
		self.result_args = self.result_args.strip ()
			
	def on_dvd_info (self):
		if sys.platform.startswith ('win'):
			tkMessageBox.showerror ("Cannot show info", "The DVD information feature does not work on Windows")
		else:
			dvdinfodlg = DVDInfoDialog (self, self.dvddevice.get())
	
	def body (self, master):
		Tkinter.Label (master, text="DVD input options (see Misc Options for choosing subtitles/audio language)", fg="darkblue", wraplength=280).grid (column=0, row=0, columnspan=3)
		
		Tkinter.Label (master, text="Title/track #").grid (column=0, row=1)
		
		self.dvdtrack = Tkinter.StringVar ()
		self.dvdtrack.set (1)
		Tkinter.Entry (master, textvariable=self.dvdtrack).grid (column=1, row=1)
		
		Tkinter.Button (master, text="View DVD info...", command=self.on_dvd_info).grid (column=2, row=1)
		
		self.allchapters = Tkinter.IntVar ()
		self.allchapters.set (1)
		Tkinter.Checkbutton (master, variable=self.allchapters, text="Encode all chapters on track", onvalue=1, offvalue=0).grid (column=0, row=2, columnspan=2)
		
		Tkinter.Label (master, text="Start chapter #").grid (column=0, row=3)
		
		self.startchapter = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.startchapter).grid (column=1, row=3)
		
		Tkinter.Label (master, text="End chapter #").grid (column=0, row=4)
		
		self.endchapter = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.endchapter).grid (column=1, row=4)
		
		Tkinter.Label (master, text="DVD device (default: /dev/dvd)").grid (column=0, row=5)
		
		self.dvddevice = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.dvddevice).grid (column=1, row=5)
		
class MiscOptionsDialog (tkSimpleDialog.Dialog):
	""" Class to configure miscellaneous mencoder specific options """
	def apply (self):
		self.result = ""
		if self.startpos.get().strip() != "":
			self.result += "-ss " + self.startpos.get()
		if self.endpos.get().strip() != "":
			self.result += " -endpos " + self.endpos.get()
		if self.resamplerate.get().strip() != "":
			self.result += " -srate " + self.resamplerate.get()
		if self.audiolang.get().strip() != "":
			self.result += " -alang " + self.audiolang.get()
		if self.subtitlelang.get().strip() != "":
			self.result += " -slang " + self.subtitlelang.get()
		if self.fourcc.get().strip() != "":
			self.result += " -ffourcc " + self.fourcc.get()
			
		self.result = self.result.strip ()
	
	def body (self, master):
		Tkinter.Label (master, text="Miscellaneous mencoder options (leave blank for defaults)", fg="darkblue").grid (column=0, row=0, columnspan=2)
		
		Tkinter.Label (master, text="Input stream options", fg="darkblue").grid (column=0, row=1, columnspan=3, pady=10)
		
		Tkinter.Label (master, text="Start encoding from (sec or hh:mm:ss)").grid (column=0, row=2)
		
		self.startpos = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.startpos).grid (column=1, row=2)
		
		Tkinter.Label (master, text="Stop encoding after (sec or hh:mm:ss or b/kb/mb)").grid (column=0, row=3)
		self.endpos = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.endpos).grid (column=1, row=3)
		
		Tkinter.Label (master, text="Audio and subtitle (only if available) options", fg="darkblue").grid (column=0, row=4, columnspan=2, pady=10)
		
		Tkinter.Label (master, text="Resample audio (Hz)").grid (column=0, row=5)
		
		self.resamplerate = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.resamplerate).grid (column=1, row=5)		
		
		Tkinter.Label (master, text="Audio language (code)").grid (column=0, row=6)
				
		self.audiolang = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.audiolang).grid (column=1, row=6)
		
		Tkinter.Label (master, text="Subtitle language (code)").grid (column=0, row=7)
		
		self.subtitlelang = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.subtitlelang).grid (column=1, row=7)
		
		Tkinter.Label (master, text="Advanced", fg="darkblue").grid (column=0, row=8, columnspan=2, pady=10)
		
		Tkinter.Label (master, text="Override video four character code").grid (column=0, row=9)
		
		self.fourcc = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.fourcc).grid (column=1, row=9)
		
class ProfileLoadDialog (tkSimpleDialog.Dialog):
	""" Class to load a profile or simply delete existing profiles """
	def apply (self):
		self.result = self.conflist.get (Tkinter.ANCHOR)
		if self.result.strip () == "":
			self.result = None

	def load_profiles (self):
		cfgfile = os.path.join (os.path.expanduser ("~"), ".biamoverc")
		config = CP.SafeConfigParser ()
		config.read (cfgfile)
		
		confsections = []
		for section in config.sections ():
			self.conflist.insert (Tkinter.END, section)
	
	def on_delete_profile (self):
		cfgfile = os.path.join (os.path.expanduser ("~"), ".biamoverc")
		config = CP.SafeConfigParser ()
		config.read (cfgfile)
		
		delsection = self.conflist.get (Tkinter.ANCHOR)
		self.conflist.delete (Tkinter.ANCHOR)
		
		if (config.has_section (delsection)):
			config.remove_section (delsection)
			config.write (file (cfgfile, "w"))
	
	def body (self, master):
		Tkinter.Label (master, text = "Load profile", fg="darkblue").grid (column=0, row=0, columnspan=2)
		
		frame = Tkinter.Frame(master)
		scrollbar = Tkinter.Scrollbar(frame, orient=Tkinter.VERTICAL)
		self.conflist = Tkinter.Listbox (frame, width=30, height=5, yscrollcommand=scrollbar.set)
		scrollbar.config(command=self.conflist.yview)
		scrollbar.pack(side=Tkinter.RIGHT, fill=Tkinter.Y)
		self.conflist.pack (side=Tkinter.LEFT, fill=Tkinter.BOTH)
		frame.grid (column=0, row=1)
		
		Tkinter.Button (master, text="delete", command = self.on_delete_profile).grid (column=1, row=1)
		
		self.load_profiles ()
