# Video filter dialogs for the BiaMovE application

import Tkinter
import tkFileDialog
import tkSimpleDialog
import tkMessageBox

class FilterPictureDialog (tkSimpleDialog.Dialog):
	"""Implements a picture dialog box for adjusting brightness, contrast etc."""
	
	def apply (self):
		self.result = self.gamma.get () + ":" + self.contrast.get () + ":" + self.brightness.get() + ":" + self.saturation.get ()
		
	def body (self, master):
		Tkinter.Label (master, text="Picture adjustments (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=2)
		
		Tkinter.Label (master, text="Gamma correction").grid (column=0, row=1)
		
		self.gamma = Tkinter.StringVar ()
		self.gamma_scale = Tkinter.Scale (master, variable=self.gamma, from_=0.1, to=10, length=200, resolution=0.05,  orient=Tkinter.HORIZONTAL)
		self.gamma_scale.set (1)
		self.gamma_scale.grid (column=1, row=1)
		
		Tkinter.Label (master, text="Contrast").grid (column=0, row=2)
		
		self.contrast = Tkinter.StringVar ()
		self.contrast_scale = Tkinter.Scale (master, variable=self.contrast, from_=-2, to=2, length=200, resolution=0.05,
		orient=Tkinter.HORIZONTAL)
		self.contrast_scale.set (1)
		self.contrast_scale.grid (column=1, row=2)
		
		Tkinter.Label (master, text="Brightness").grid (column=0, row=3)
		
		self.brightness = Tkinter.StringVar ()
		self.brightness_scale = Tkinter.Scale (master, variable=self.brightness, from_=-1, to=1, length=200,
		resolution=0.05, orient=Tkinter.HORIZONTAL)
		self.brightness_scale.set (0)
		self.brightness_scale.grid (column=1, row=3)
		
		Tkinter.Label (master, text="Saturation").grid (column=0,row=4)
		
		self.saturation = Tkinter.StringVar ()
		self.saturation_scale = Tkinter.Scale (master, variable=self.saturation, from_=0, to=3, length=200,
		resolution=0.05, orient=Tkinter.HORIZONTAL)
		self.saturation_scale.set (1)
		self.saturation_scale.grid (column=1, row=4)

class FilterCropDialog (tkSimpleDialog.Dialog):
	"""Implements the crop dialog box for cropping parts of the video"""
	def apply (self):
		self.result = self.crop_width.get() + ":" + self.crop_height.get()
		
		self.result += ":"
		
		if self.center_width.get() == 0:
			self.result += self.horizontal.get ()
		self.result += ":"
		if self.center_height.get() == 0:
			self.result += self.vertical.get ()
		
		self.result = self.result.strip (":")
	
	def body (self, master):
		Tkinter.Label (master, text="Crop parameters (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=2)
		Tkinter.Label (master, text="Width").grid (column=0, row=1)
		
		self.crop_width = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.crop_width).grid (column=1, row=1)
		
		Tkinter.Label (master, text="Height").grid (column=0, row=2)
		
		self.crop_height = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.crop_height).grid (column=1, row=2)
		
		Tkinter.Label (master, text="Position of video in cropped frame", fg="darkblue").grid (column=0, row=3, columnspan=2)
		
		self.center_width = Tkinter.IntVar ()
		self.center_width.set (1)
		Tkinter.Checkbutton (master, variable=self.center_width, text="Center horizontally", onvalue=1, offvalue=0).grid (column=0, row=4)
		
		self.center_height = Tkinter.IntVar ()
		self.center_height.set (1)
		Tkinter.Checkbutton (master, variable=self.center_height, text="Center vertically", onvalue=1, offvalue=0).grid (column=1, row=4)
		
		Tkinter.Label (master, text="Horizontal position").grid (column=0, row=5)
		
		self.horizontal = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.horizontal).grid (column=1, row=5)
		
		Tkinter.Label (master, text="Vertical position").grid (column=0, row=6)
		
		self.vertical = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.vertical).grid (column=1, row=6)

class FilterExpandDialog (tkSimpleDialog.Dialog):
	"""Implements the expand dialog box for expanding the video"""
	def apply (self):
		if self.relative_width.get() == 1:
			self.result = "-" + self.expand_width.get()
		else:
			self.result = self.expand_width.get()
		if self.relative_height.get() == 1:
			self.result += ":-" + self.expand_height.get()
		else:
			self.result += ":" + self.expand_height.get()
		
		self.result += ":"
		if self.center_width.get() == 0:
			self.result += self.horizontal.get()
		self.result += ":"
		if self.center_height.get() == 0:
			self.result += self.vertical.get()
			
		self.result += ":" + str (self.osdsubtitle.get())
		
	
	def body (self, master):
		Tkinter.Label (master, text="Expand parameters (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=3)
		Tkinter.Label (master, text="Width").grid (column=0, row=1)
		
		self.expand_width = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.expand_width).grid (column=1, row=1)
		
		self.relative_width = Tkinter.IntVar ()
		self.relative_width.set (0)
		Tkinter.Checkbutton (master, variable = self.relative_width, text="Offset width (not absolute)", onvalue=1, offvalue=0).grid (column=2, row=1)
		
		Tkinter.Label (master, text="Height").grid (column=0, row=2)
		
		self.expand_height = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.expand_height).grid (column=1, row=2)
		
		self.relative_height = Tkinter.IntVar ()
		self.relative_height.set (0)
		Tkinter.Checkbutton (master, variable = self.relative_height, text="Offset height (not absolute)", onvalue=1, offvalue=0).grid (column=2, row=2)
		
		Tkinter.Label (master, text="Position of video in expanded frame", fg="darkblue").grid (column=0, row=3, columnspan=3)
		
		self.center_width = Tkinter.IntVar ()
		self.center_width.set (1)
		Tkinter.Checkbutton (master, variable=self.center_width, text="Center horizontally", onvalue=1, offvalue=0).grid (column=1, row=4)
		
		self.center_height = Tkinter.IntVar ()
		self.center_height.set (1)
		Tkinter.Checkbutton (master, variable=self.center_height, text="Center vertically", onvalue=1, offvalue=0).grid (column=2, row=4)
		
		Tkinter.Label (master, text="Horizontal position").grid (column=0, row=5)
		
		self.horizontal = Tkinter.StringVar ()
		self.horizontal.set (0)
		Tkinter.Entry (master, textvariable=self.horizontal).grid (column=1, row=5)
		
		Tkinter.Label (master, text="Vertical position").grid (column=0, row=6)
		
		self.vertical = Tkinter.StringVar ()
		self.vertical.set (0)
		Tkinter.Entry (master, textvariable=self.vertical).grid (column=1, row=6)
		
		self.osdsubtitle = Tkinter.IntVar ()
		self.osdsubtitle.set (0)
		Tkinter.Checkbutton (master, variable=self.osdsubtitle, text="OSD/Subtitle rendering", onvalue=1, offvalue=0).grid (column=0, row=7, columnspan=3)

class FilterScaleDialog (tkSimpleDialog.Dialog):
	"""Implements the dialog box for scaling the video"""
	def apply (self):
		if self.width_mode.get() == "(specify manually)":
			self.result = self.width_value.get()
		else:
			self.result = self.width_mode.get().split()[0]
		if self.height_mode.get() == "(specify manually)":
			self.result += ":" + self.height_value.get()
		else:
			self.result += ":" + self.height_mode.get().split()[0]
		self.result += ":" + str(self.interlacing.get())
		
	def body (self, master):
		Tkinter.Label (master, text="Video scale parameters (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=3)
		
		Tkinter.Label (master, text="Width").grid (column=0, row=1)
		
		self.width_mode = Tkinter.StringVar ()
		self.width_mode.set ("(specify manually)")
		Tkinter.OptionMenu (master, self.width_mode, "0   scaled d_width", "-1   original width", "-2   calculate from height using prescaled aspect ratio", "-3   calculate from height using original aspect ratio", "(specify manually)").grid (column=1, row=1)
		
		self.width_value = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.width_value).grid (column=2, row=1)
		
		Tkinter.Label (master, text="Height").grid (column=0, row=2)
		
		self.height_mode = Tkinter.StringVar ()
		self.height_mode.set ("(specify manually)")
		Tkinter.OptionMenu (master, self.height_mode, "0   scaled d_height", "-1   original height", "-2   calculate from width using prescaled aspect ratio", "-3   calculate from width using original aspect ratio", "(specify manually)").grid (column=1, row=2)
		
		self.height_value = Tkinter.StringVar ()
		Tkinter.Entry (master, textvariable=self.height_value).grid (column=2, row=2)
		
		self.interlacing = Tkinter.IntVar ()
		self.interlacing.set (0)
		Tkinter.Checkbutton (master, variable=self.interlacing, text="Interaced scaling", onvalue=1, offvalue=0).grid (column=0, row=3, columnspan=3)

class VideoFiltersDialog (tkSimpleDialog.Dialog):
	""" Implement a dialog box to implement video filter
	1. crop
	2. scale
	3. expand
	4. force encoding duplicate frames (harddup)
	5. picture adjustments (brightness contrast etc)"""
	def on_crop_filter (self):
		cropdlg = FilterCropDialog (self)
		if cropdlg.result != None:
			self.crop_params.set (cropdlg.result)
			self.active_filters["crop"] = cropdlg.result
			
			if "crop" not in self.filterlist.get(0, Tkinter.END):
				self.filterlist.insert (Tkinter.END, "crop")
			
	def on_scale_filter (self):
		scaledlg = FilterScaleDialog (self)
		if scaledlg.result != None:
			self.scale_params.set (scaledlg.result)
			self.active_filters["scale"] = scaledlg.result
			
			if "scale" not in self.filterlist.get(0, Tkinter.END):
				self.filterlist.insert (Tkinter.END, "scale")
				
	def on_remove (self):
		self.filterlist.delete (Tkinter.ANCHOR)
		
	def on_softskip_filter (self):
		if "softskip" not in self.filterlist.get (0, Tkinter.END):
			self.filterlist.insert (Tkinter.END, "softskip")
			
	def on_expand_filter (self):
		expanddlg = FilterExpandDialog (self)
		if expanddlg.result != None:
			self.expand_params.set (expanddlg.result)
			self.active_filters["expand"] = expanddlg.result
			
			if "expand" not in self.filterlist.get (0, Tkinter.END):
				self.filterlist.insert (Tkinter.END, "expand")
				
	def on_picture_filter (self):
		picturedlg = FilterPictureDialog (self)
		if picturedlg.result != None:
			self.picture_params.set (picturedlg.result)
			self.active_filters["eq2"] = picturedlg.result
			
			if "eq2" not in self.filterlist.get (0, Tkinter.END):
				self.filterlist.insert (Tkinter.END, "eq2")
	
	def apply (self):
		self.result = ""
		for item in self.filterlist.get (0, Tkinter.END):
			if self.active_filters.has_key (item):
				self.result += item + "=" + self.active_filters[item]
			else:
				self.result += item
			self.result += ","

		if self.harddup.get () == 1:
			self.result += "harddup"
		self.result = self.result.strip (",")

	def body (self, master):
		Tkinter.Label (master, text="Setup video filters (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=4)
		
		Tkinter.Label (master, text="Active filters list").grid (column=0, row=1)
		self.filterlist = Tkinter.Listbox (master, height=5,width=30)
		self.filterlist.grid (column=1, row=1, rowspan=6)
		Tkinter.Button (master, text="remove", command=self.on_remove).grid(column=1,row=1)
		
		Tkinter.Button (master, text="crop...", command=self.on_crop_filter).grid (column=2, row=2)
		
		self.crop_params = Tkinter.StringVar ()
		Tkinter.Label (master, textvariable=self.crop_params).grid (column=3, row=2)
		
		Tkinter.Button (master, text="scale...", command=self.on_scale_filter).grid (column=2, row=3)
		
		self.scale_params = Tkinter.StringVar ()
		Tkinter.Label (master, textvariable=self.scale_params).grid (column=3, row=3)
		
		Tkinter.Button (master, text="expand...", command=self.on_expand_filter).grid (column=2, row=4)
		
		self.expand_params = Tkinter.StringVar ()
		Tkinter.Label (master, textvariable=self.expand_params).grid (column=3, row=4)
		
		Tkinter.Button (master, text="picture...", command=self.on_picture_filter).grid (column=2, row=5)
		self.picture_params = Tkinter.StringVar ()
		Tkinter.Label (master, textvariable=self.picture_params).grid (column=3, row=5)
		
		Tkinter.Button (master, text="softskip", command=self.on_softskip_filter).grid (column=2, row=6)
		
		self.harddup = Tkinter.IntVar ()
		self.harddup.set (0)
		Tkinter.Checkbutton (master, variable=self.harddup, text="Force duplicate frames (required for DVD-compliant MPEG-2 streams)", onvalue=1, offvalue=0).grid (column=0, row=7, columnspan=4)
		
		self.active_filters = {}
