# Video muxer dialogs for the BiaMovE application

import Tkinter
import tkFileDialog
import tkSimpleDialog
import tkMessageBox

class MPEGMuxerDialog (tkSimpleDialog.Dialog):
	"""MPEG container -mpegopts options dialog"""
	def apply (self):
		self.result = "-mpegopts format=" + self.format.get ()
		if (self.vdelay.get() != "0"):
			self.result += ":vdelay=" + self.vdelay.get ()
			if (self.vdropaudio.get() == 1):
				self.result += ":drop"
		if (self.adelay.get() != "0"):
			self.result += ":adelay=" + self.adelay.get ()
		if (self.aspectratio.get() != "(not set)"):
			self.result += ":vaspect=" + self.aspectratio.get ()
		if (self.framerate.get() != "(not set)"):
			self.result += ":vframerate=" + self.framerate.get ()
		if (self.bitrate.get() != "0"):
			self.result += ":vbitrate=" + self.bitrate.get ()
		if (self.settsaf.get() == 1):
			self.result += ":tsaf"
		if (self.interleaving.get() == 1):
			self.result += ":interleaving2"

	def body (self, master):
		Tkinter.Label (master, text="MPEG muxer options (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=3)
		
		Tkinter.Label (master, text="Format").grid (column=0, row=1)
		
		self.format = Tkinter.StringVar ()
		self.format.set ("mpeg2")
		Tkinter.OptionMenu (master, self.format, "mpeg1", "mpeg2", "xvcd", "xsvcd", "dvd", "pes1", "pes2").grid (column=1, row=1)
		
		Tkinter.Label (master, text="Video delay (1 to 32760) ms").grid (column=0, row=2)
		
		self.vdelay = Tkinter.StringVar ()
		self.vdelay.set (0)
		Tkinter.Spinbox (master, textvariable=self.vdelay, from_=1, to=32760).grid (column=1, row=2)
		
		self.vdropaudio = Tkinter.IntVar ()
		self.vdropaudio.set (0)
		Tkinter.Checkbutton (master, text = "Drop leading audio", variable=self.vdropaudio, offvalue=0, onvalue=1).grid (column=2, row=2)
		
		Tkinter.Label (master, text="Audio delay (1 to 32760) ms").grid (column=0, row=3)
		
		self.adelay = Tkinter.StringVar ()
		self.adelay.set (0)
		Tkinter.Spinbox (master, textvariable=self.adelay, from_=1, to=32760).grid (column=1, row=3)
		
		Tkinter.Label (master, text="Aspect ratio").grid (column=0, row=4)
		
		self.aspectratio = Tkinter.StringVar ()
		self.aspectratio.set ("(not set)")
		Tkinter.OptionMenu (master, self.aspectratio, "(not set)", "1", "4/3", "16/9", "221/100").grid (column=1, row=4)
		
		Tkinter.Label (master, text="Frame rate").grid (column=0, row=5)
		
		self.framerate = Tkinter.StringVar ()
		self.framerate.set ("(not set)")
		Tkinter.OptionMenu (master, self.framerate, "(not set)", "24000/1001", "24", "25", "30000/1001", "30", "50", "60000/1001", "60").grid (column=1, row=5)
		
		Tkinter.Label (master, text="Video bitrate (0 to ignore)").grid (column=0, row=6)
		
		self.bitrate = Tkinter.StringVar ()
		self.bitrate.set (0)
		Tkinter.Spinbox (master, textvariable=self.bitrate, from_=0, to=16000).grid (column=1, row=6)
		
		self.settsaf = Tkinter.IntVar ()
		self.settsaf.set (0)
		Tkinter.Checkbutton (master, text="Set timestamp on all frames",variable=self.settsaf, onvalue=1, offvalue=0).grid (column=0, row=7, columnspan=3)
		
		self.interleaving = Tkinter.IntVar ()
		self.interleaving.set (0)
		Tkinter.Checkbutton (master, text="Interleaving 2", variable=self.interleaving, onvalue=1, offvalue=0).grid (column=0, row=8, columnspan=3)

class LAVFMuxerDialog (tkSimpleDialog.Dialog):
	"""LAVF container -lavfopts options dialog"""
	def apply (self):
		if self.containerfmt.get () == "(autodetect from filename)":
			self.result = ""
		else:
			self.result = "-lavfopts format=" + self.containerfmt.get().split()[0]
	
	def body (self, master):
		Tkinter.Label(master, text="libavformat muxer options (simplified)", fg="darkblue").grid (column=0, row=0, columnspan=2)
		
		Tkinter.Label(master, text="Container format").grid (column=0, row=1)
		
		self.containerfmt = Tkinter.StringVar ()
		self.containerfmt.set ("(autodetect from filename)")
		Tkinter.OptionMenu (master, self.containerfmt, "(autodetect from filename)", "mpg   MPEG-1 systems or MPEG-2 PS", "asf   Advanced streaming format", "avi   Audio-video interleaved", "wav   Waveform audio", "swf   Macromedia Flash", "flv   Macromedia Flash video", "rm   Realaudio/Realvideo", "nut   NUT open container format (experimental)", "mov   QuickTime", "mp4   MPEG-4 format", "dv   Sony digital video container").grid (column=1, row=1)
